@@ -49,6 +49,8 @@
 #include "gvfsjobqueryattributes.h"
 #include "gvfsjobenumerate.h"
 #include "gvfsdaemonprotocol.h"
+#include "gvfsdaemonutils.h"
+#include "gvfsutils.h"
 #include "gvfskeyring.h"
 
 #include <libsmbclient.h>
@@ -84,6 +86,7 @@ struct _GVfsBackendSmb
   int mount_try;
   gboolean mount_try_again;
   gboolean mount_cancelled;
+  gboolean use_anonymous;
 	
   gboolean password_in_keyring;
   GPasswordSave password_save;
@@ -176,7 +179,7 @@ auth_callback (SMBCCTX *context,
 {
   GVfsBackendSmb *backend;
   char *ask_password, *ask_user, *ask_domain;
-  gboolean handled, abort;
+  gboolean handled, abort, anonymous = FALSE;
 
   backend = smbc_getOptionUserData (context);
 
@@ -213,9 +216,16 @@ auth_callback (SMBCCTX *context,
       backend->user == NULL &&
       backend->domain == NULL)
     {
-      /* Try again if kerberos login + anonymous fallback fails */
+      /* Try again if kerberos login fails */
       backend->mount_try_again = TRUE;
-      DEBUG ("auth_callback - anonymous pass\n");
+      DEBUG ("auth_callback - kerberos pass\n");
+    }
+  else if (backend->use_anonymous)
+    {
+      /* Try again if anonymous login fails */
+      backend->use_anonymous = FALSE;
+      backend->mount_try_again = TRUE;
+      DEBUG ("auth_callback - anonymous login pass\n");
     }
   else
     {
@@ -254,6 +264,8 @@ auth_callback (SMBCCTX *context,
 	    flags |= G_ASK_PASSWORD_NEED_DOMAIN;
 	  if (backend->user == NULL)
 	    flags |= G_ASK_PASSWORD_NEED_USERNAME;
+          if (backend->user == NULL && backend->domain == NULL)
+	    flags |= G_ASK_PASSWORD_ANONYMOUS_SUPPORTED;
 
 	  DEBUG ("auth_callback - asking for password...\n");
 
@@ -269,7 +281,7 @@ auth_callback (SMBCCTX *context,
 						 &ask_password,
 						 &ask_user,
 						 &ask_domain,
-						 NULL,
+						 &anonymous,
 						 &(backend->password_save));
 	  g_free (message);
 	  if (!handled)
@@ -287,11 +299,19 @@ auth_callback (SMBCCTX *context,
       /* Try again if this fails */
       backend->mount_try_again = TRUE;
 
-      strncpy (password_out, ask_password, pwmaxlen);
-      if (ask_user && *ask_user)
-	strncpy (username_out, ask_user, unmaxlen);
-      if (ask_domain && *ask_domain)
-	strncpy (domain_out, ask_domain, domainmaxlen);
+      if (anonymous)
+        {
+          backend->use_anonymous = TRUE;
+          backend->password_save = FALSE;
+        }
+      else
+        {
+          strncpy (password_out, ask_password, pwmaxlen);
+          if (ask_user && *ask_user)
+            strncpy (username_out, ask_user, unmaxlen);
+          if (ask_domain && *ask_domain)
+            strncpy (domain_out, ask_domain, domainmaxlen);
+        }
 
     out:
       g_free (ask_password);
@@ -551,8 +571,7 @@ do_mount (GVfsBackend *backend,
   smbc_setOptionUseKerberos (smb_context, 1);
   smbc_setOptionFallbackAfterKerberos (smb_context,
                                        op_backend->user != NULL);
-  smbc_setOptionNoAutoAnonymousLogin (smb_context,
-                                      op_backend->user != NULL);
+  smbc_setOptionNoAutoAnonymousLogin (smb_context, TRUE);
 
   
 #if 0
@@ -636,15 +655,20 @@ do_mount (GVfsBackend *backend,
         }
 
       /* The first round is Kerberos-only.  Only if this fails do we enable
-       * NTLMSSP fallback (turning off anonymous fallback, which we've
-       * already tried and failed with).
+       * NTLMSSP fallback.
        */
       if (op_backend->mount_try == 0)
         {
           DEBUG ("do_mount - after anon, enabling NTLMSSP fallback\n");
           smbc_setOptionFallbackAfterKerberos (op_backend->smb_context, 1);
-          smbc_setOptionNoAutoAnonymousLogin (op_backend->smb_context, 1);
         }
+
+      /* If the AskPassword reply requested anonymous login, enable the
+       * anonymous fallback and try again.
+       */
+      smbc_setOptionNoAutoAnonymousLogin (op_backend->smb_context,
+                                          !op_backend->use_anonymous);
+
       op_backend->mount_try ++;
     }
   while (op_backend->mount_try_again);
@@ -792,7 +816,6 @@ do_open_for_read (GVfsBackend *backend,
       
       smbc_stat = smbc_getFunctionStat (op_backend->smb_context);
       res = smbc_stat (op_backend->smb_context, uri, &st);
-      g_free (uri);
       if ((res == 0) && (S_ISDIR (st.st_mode)))
             g_vfs_job_failed (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_IS_DIRECTORY,
@@ -807,6 +830,7 @@ do_open_for_read (GVfsBackend *backend,
       g_vfs_job_open_for_read_set_handle (job, file);
       g_vfs_job_succeeded (G_VFS_JOB (job));
     }
+  g_free (uri);
 }
 
 static void
@@ -852,18 +876,8 @@ do_seek_on_read (GVfsBackend *backend,
   off_t res;
   smbc_lseek_fn smbc_lseek;
 
-  switch (type)
+  if ((whence = gvfs_seek_type_to_lseek (type)) == -1)
     {
-    case G_SEEK_SET:
-      whence = SEEK_SET;
-      break;
-    case G_SEEK_CUR:
-      whence = SEEK_CUR;
-      break;
-    case G_SEEK_END:
-      whence = SEEK_END;
-      break;
-    default:
       g_vfs_job_failed (G_VFS_JOB (job),
 			G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
 			_("Unsupported seek type"));
@@ -1030,16 +1044,6 @@ do_append_to (GVfsBackend *backend,
 }
 
 
-static void
-random_chars (char *str, int len)
-{
-  int i;
-  const char chars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
-
-  for (i = 0; i < len; i++)
-    str[i] = chars[g_random_int_range (0, strlen(chars))];
-}
-
 static char *
 get_dir_from_uri (const char *uri)
 {
@@ -1073,7 +1077,7 @@ open_tmpfile (GVfsBackendSmb *backend,
   dir_uri = get_dir_from_uri (uri);
  
   do {
-    random_chars (filename + 4, 4);
+    gvfs_randomize_string (filename + 4, 4);
     tmp_uri = g_strconcat (dir_uri, filename, NULL);
 
     smbc_open = smbc_getFunctionOpen (backend->smb_context);
@@ -1351,18 +1355,8 @@ do_seek_on_write (GVfsBackend *backend,
   off_t res;
   smbc_lseek_fn smbc_lseek;
 
-  switch (type)
+  if ((whence = gvfs_seek_type_to_lseek (type)) == -1)
     {
-    case G_SEEK_SET:
-      whence = SEEK_SET;
-      break;
-    case G_SEEK_CUR:
-      whence = SEEK_CUR;
-      break;
-    case G_SEEK_END:
-      whence = SEEK_END;
-      break;
-    default:
       g_vfs_job_failed (G_VFS_JOB (job),
 			G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
 			_("Unsupported seek type"));
